#include <array.au3>
#include <date.au3>
#include "XMLWrapperEx.au3"


;~ _Example_1__XML_SelectNodes()
;~ _Example_2__XML_CreateChildWAttr()
;~ _Example_3__XML_ErrorParser_GetDescription()
;~ _Example_MSDN_1__setAttributeNode()



Func _Example_1__XML_SelectNodes()
	_XML_MiscProperty_NotifyAll(True)
	_XML_MiscProperty_NotifyToMsgBox(False)

	Local $oErrorHandler = ObjEvent("AutoIt.Error", "_ErrFunc_CustomUserHandler")

	; Add user ComError handler ; it will be used toogether with internal ComError handler
	_XML_ComErrorHandler_UserFunction(_ErrFunc_CustomUserHandler)

	Local $sXmlFile = @ScriptDir & "\XML_Example_XML_Files\MSDN\books.xml"
	Local $oXmlDoc = _XML_CreateObject()
	_XML_Load($oXmlDoc, $sXmlFile)
	MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, '_Example_1__XML_SelectNodes', _XML_TIDY($oXmlDoc))

	Local $oNodesColl = _XML_SelectNodes($oXmlDoc, "//price")
	Local $aNodesColl = _XML_Array_GetNodesProperties($oNodesColl)
	_ArrayDisplay($aNodesColl, '_Example_1__XML_SelectNodes : ' & 'Length=' & $oNodesColl.length & '   XPath=' & $oNodesColl.expr)

EndFunc   ;==>_Example_1__XML_SelectNodes

Func _Example_2__XML_CreateChildWAttr()
	_XML_MiscProperty_NotifyAll(True)
	_XML_MiscProperty_NotifyToMsgBox(False)

	Local $oErrorHandler = ObjEvent("AutoIt.Error", "_ErrFunc_CustomUserHandler")

	; Add user ComError handler ; it will be used toogether with internal ComError handler
	_XML_ComErrorHandler_UserFunction(_ErrFunc_CustomUserHandler)

	Local $sXmlFile = @ScriptDir & "\XML_Example_XML_Files\MSDN\books.xml"
	Local $oXmlDoc = _XML_CreateObject()
	_XML_Load($oXmlDoc, $sXmlFile)
	MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, 'Before _Example_2__XML_CreateChildWAttr()', _XML_TIDY($oXmlDoc))


	Local $aAttributeList[2][2] = [['Fist', '1'], ['Second', '2']]
	_XML_CreateChildWAttr($oXmlDoc, "//book", 'MyNewNode', $aAttributeList, "SomeText")

	; TIDY xml and save result back to object
	MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, 'After _Example_2__XML_CreateChildWAttr()', _XML_TIDY($oXmlDoc))
	$oXmlDoc.loadXML(_XML_TIDY($oXmlDoc))
	_XML_Misc_Viewer($oXmlDoc)

EndFunc   ;==>_Example_2__XML_CreateChildWAttr

Func _Example_3__XML_ErrorParser_GetDescription()
	_XML_MiscProperty_NotifyAll(True)
	_XML_MiscProperty_NotifyToMsgBox(False)

	Local $oErrorHandler = ObjEvent("AutoIt.Error", "_ErrFunc_CustomUserHandler")

	; Add user ComError handler ; it will be used toogether with internal ComError handler
	_XML_ComErrorHandler_UserFunction(_ErrFunc_CustomUserHandler)

	Local $sXmlFile = @ScriptDir & "\XML_Example_XML_Files\MSDN\books.xml"
	Local $oXmlDoc = _XML_CreateObject()
	_XML_Load($oXmlDoc, $sXmlFile)

	Local $oNodesColl = _XML_SelectNodes($oXmlDoc, "/1/price")
	#forceref $oNodesColl
	If @error Then
		MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, @error & ' ' & @extended, _XML_Misc_ErrorParser(@error, @extended))
	EndIf


	; now we fires $oXmlDoc.parseError.errorCode - wrong XML file
	$sXmlFile = @ScriptFullPath
	$oXmlDoc = _XML_CreateObject()
	_XML_Load($oXmlDoc, $sXmlFile)
	If @error = $XMLWRAPPER_ERROR_PARSE Then
		MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, @error & ' ' & @extended & ' _XML_ErrorParser_GetDescription', _XML_ErrorParser_GetDescription($oXmlDoc))
	EndIf

EndFunc   ;==>_Example_3__XML_ErrorParser_GetDescription

Func _Example_MSDN_1__setAttributeNode()
	_XML_MiscProperty_NotifyAll(True)
	_XML_MiscProperty_NotifyToMsgBox(False)

	Local $oErrorHandler = ObjEvent("AutoIt.Error", "_ErrFunc_CustomUserHandler")

	; Add user ComError handler ; it will be used toogether with internal ComError handler
	_XML_ComErrorHandler_UserFunction(_ErrFunc_CustomUserHandler)

	Local $sXmlFile = @ScriptDir & "\XML_Example_XML_Files\MSDN\books.xml"
	Local $oXmlDoc = _XML_CreateObject()
	_XML_Load($oXmlDoc, $sXmlFile)
	MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, '_Example_MSDN_1__setAttributeNode', _XML_TIDY($oXmlDoc))

	;MSDN; https://msdn.microsoft.com/en-us/library/ms762235(v=vs.85).aspx

	;MSDN; nodePublishDate = xmlDoc.createAttribute("PublishDate");
	Local $oNodePublishDate = $oXmlDoc.createAttribute("PublishDate");

	; Checking
	MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, 'Temp', _XML_TIDY($oXmlDoc)) ; Hmmm... Nothing happend

	;MSDN; nodePublishDate.value = String(Date());
	$oNodePublishDate.value = String(_Now());

	;MSDN; nodeBook = xmlDoc.selectSingleNode("//book");
	Local $oNodeBook = $oXmlDoc.selectSingleNode("//book");

	;MSDN; nodeBook.setAttributeNode(nodePublishDate);
	$oNodeBook.setAttributeNode($oNodePublishDate);

	;MSDN; WScript.Echo(nodeBook.getAttribute("PublishDate"));
	ConsoleWrite($oNodeBook.getAttribute("PublishDate") & @CRLF)

	MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, 'Result', _XML_TIDY($oXmlDoc))

	; this following part is not related to MSDN but is usefull to show how _XML_Array_GetNodesProperties display attributes
	Local $oNodesColl = _XML_SelectNodes($oXmlDoc, "//book")
	Local $aNodesColl = _XML_Array_GetNodesProperties($oNodesColl)
	_ArrayDisplay($aNodesColl, '_Example_1__XML_SelectNodes : ' & $oNodesColl.expr)

EndFunc   ;==>_Example_MSDN_1__setAttributeNode

Func _ErrFunc_CustomUserHandler($oError)
	ConsoleWrite(@ScriptName & " (" & $oError.scriptline & ") : ==> COM Error intercepted !" & @CRLF & _
			@TAB & "err.number is: " & @TAB & @TAB & "0x" & Hex($oError.number) & @CRLF & _
			@TAB & "err.windescription:" & @TAB & $oError.windescription & @CRLF & _
			@TAB & "err.description is: " & @TAB & $oError.description & @CRLF & _
			@TAB & "err.source is: " & @TAB & @TAB & $oError.source & @CRLF & _
			@TAB & "err.helpfile is: " & @TAB & $oError.helpfile & @CRLF & _
			@TAB & "err.helpcontext is: " & @TAB & $oError.helpcontext & @CRLF & _
			@TAB & "err.lastdllerror is: " & @TAB & $oError.lastdllerror & @CRLF & _
			@TAB & "err.scriptline is: " & @TAB & $oError.scriptline & @CRLF & _
			@TAB & "err.retcode is: " & @TAB & "0x" & Hex($oError.retcode) & @CRLF & @CRLF)
EndFunc   ;==>_ErrFunc_CustomUserHandler


; #FUNCTION# ====================================================================================================================
; Name ..........: _XML_Misc_ErrorDecription
; Description ...: Changing $XMLWRAPPER_ERROR_... to human readable description
; Syntax ........: _XML_Misc_ErrorDecription($iXMLWrapper_Error, $iExtended)
; Parameters ....: $iXMLWrapper_Error	- an integer value.
;                  $iExtended           - an integer value.
; Return values .: None
; Author ........: mLipok
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _XML_Misc_ErrorParser($iXMLWrapper_Error, $iExtended)
	Switch $iXMLWrapper_Error
		Case $XMLWRAPPER_ERROR_OK
			Return 'All is ok.'
		Case $XMLWRAPPER_ERROR_GENERAL
			Return 'The error which is not specifically defined.'
		Case $XMLWRAPPER_ERROR_ISNOTOBJECT
			Return 'No object passed to function'
		Case $XMLWRAPPER_ERROR_ISNOTVALIDOBJECT
			Return 'Invalid object passed to function'
		Case $XMLWRAPPER_ERROR_PARSE
			Return 'Error: with Parsing objects, .parseError.errorCode=' & $iExtended
		Case $XMLWRAPPER_ERROR_LOAD
			Return 'Error opening specified file.'
		Case $XMLWRAPPER_ERROR_SAVE
			Return 'Error saving file.'
		Case $XMLWRAPPER_ERROR_SAVEFILERO
			Return 'Destination file is Read Only.'
		Case $XMLWRAPPER_ERROR_XPATH
			Return 'XPath syntax error - check also COM Error Handler.'
		Case $XMLWRAPPER_ERROR_DOMVERSION
			Return 'DOM Version: ' & 'MSXML Version ' & $iExtended & ' or greater required for this function'
		Case $XMLWRAPPER_ERROR_PARAMETER
			Return 'Wrong parameter passed to function.'
		Case $XMLWRAPPER_ERROR_ARRAY
			Return 'Wrong array parameter passed to function. Check array dimension and conent.'
		Case $XMLWRAPPER_ERROR_NODECREATEAPPEND
			Return 'Can not create or append Node'
		Case Else
			Return 'NO ERROR DESCRIPTION FOR:' & $iXMLWrapper_Error
	EndSwitch
EndFunc   ;==>_XML_Misc_ErrorParser
